import {AlignX, AlignY, Point, Rect, Size} from "./engine/datatypes.ts";
import {D} from "./engine/public.ts";

let margin = 8;

export function getLayoutRect(
  size: Size,
  options?: {alignX?: AlignX, alignY?: AlignY}
): Rect {
  let {w: screenW, h: screenH} = D.size;

  // first of all: place the _internal_ screen inside the real screen
  let marginalScreenW = screenW - margin * 2;
  let marginalScreenH = screenH - margin * 2;
  let marginalScreenX = margin;
  let marginalScreenY = margin;

  // NOTE: If the screen is too small, remainingSpace will be negative
  // This is fine -- it actually results in reasonable outcomes except
  // that the size of the box is exceeded in the opposite of the align direction.
  let {w: innerW, h: innerH} = size;
  let remainingSpaceX = marginalScreenW - innerW;
  let remainingSpaceY = marginalScreenH - innerH;

  let alignXCoef =
    options?.alignX == AlignX.Left ? 0.0 :
    options?.alignX == AlignX.Center ? 0.5 :
    options?.alignX == AlignX.Right ? 1.0 :
    0.5;
  let alignYCoef =
    options?.alignY == AlignY.Top ? 0.0 :
    options?.alignY == AlignY.Middle ? 0.5 :
    options?.alignY == AlignY.Bottom ? 1.0 :
    0.5;

  let x = marginalScreenX + alignXCoef * remainingSpaceX;
  let y = marginalScreenY + alignYCoef * remainingSpaceY;

  return new Rect(
    new Point(Math.floor(x), Math.floor(y)),
    size
  )
}